#!/usr/bin/perl
#--------------------------------------
# "ANY"-to-STK Perl script
# Gregory Reed
# February 6, 2007

# Define simple STK connection wrapper function to allow for diagnosing problems in STK
	sub STK_Connect_Command
	{
		$connection=$_[0];
		$command=$_[1];
		$output=$_[2];
		
		$return=0;
		if (Send_Command_to_STK($connection, $command) eq "ACK")
		{
			$return=1;
		}
		
		# If $output variable is true, print the command to the screen
			if ($output)
			{
				print "\t" . $command;
				if ($return==0)
				{
					print " (NACK)";
				}
				print "\n";
			}
		return $return;
	}

# Print program information
	print "\"ANY\"-to-STK Perl script - Gregory Reed (reedgs@uah.edu)\n";
	print "---------------------------------------------\n";

# If no ARGV[$i] specified, print usage and exit
	if ($#ARGV==-1)
	{
		print "Usage:  gperlstk.pl <filename> [OPTION...]\n";
		print "  <filename>:  File with Maveric, OTIS, or POST data.\n";
		
		print "\n  -t TIMESTEP, --timestep=TIMESTEP\tSkip entries between timesteps (provide";
			print "\n\t\t\t\t\tin seconds)";
		print "\n  -l TIME, --limit=TIME\t\t\tPresent only up to the specified time";
			print "\n\t\t\t\t\t(provide in seconds)";
		print "\n  -h HOST, --host=HOST\t\t\tConnect to a remote STK session at the";
			print "\n\t\t\t\t\tIP or hostname provided (defaults to";
			print "\n\t\t\t\t\tlocalhost)";
		print "\n  -e PATH, --ephemeris=PATH\t\tOutput an ephemeris file to the";
			print "\n\t\t\t\t\tspecified location";
		print "\n  -a PATH, --attitude=PATH\t\tOutput an attitude file to the";
			print "\n\t\t\t\t\tspecified location";
		print "\n  -i PATH, --initialize=PATH\t\tRead STK Connect commands from the";
			print "\n\t\t\t\t\tspecified file--these are sent to STK";
			print "\n\t\t\t\t\tbefore the input file is processed";
		print "\n  -m PATH, --model=PATH\t\t\tUse the CAD model file at the specified";
			print "\n\t\t\t\t\tlocation, usually .mdl";
		print "\n  -v, --verbose\t\t\t\tPrint STK commands/responses and more";
			print "\n\t\t\t\t\tscript output to screen";
		print "\n\n";
		print "To log the output to a text file, append '> [PATH]' to the command, where [PATH]";
		print "is the full desired pathname for the output file.";
		print "\n\n";
		
		exit;
	}
	$file=$ARGV[0];
	
# Process command line arguments
	# Initialize arguments
		$timestep_skip=0;
		$time_limit=0;
		$verbose=false;
		$host="localhost";
	
	for ($i=0; $i<@ARGV; $i++)
	{
		# Timestep
			if ($ARGV[$i]=~/--timestep=*/)
			{
				$ARGV[$i]=~s/--timestep=//;
				$timestep_skip=$ARGV[$i];
			}
			elsif ($ARGV[$i] eq "-t")
			{
				$i++;
				$timestep_skip=$ARGV[$i];
			}
		# Time limit
			elsif ($ARGV[$i]=~/--limit=*/)
			{
				$ARGV[$i]=~s/--limit=//;
				$time_limit=$ARGV[$i];
			}
			elsif ($ARGV[$i] eq "-l")
			{
				$i++;
				$time_limit=$ARGV[$i];
			}
		# Host
			elsif ($ARGV[$i]=~/--host=*/)
			{
				$ARGV[$i]=~s/--host=//;
				$host=$ARGV[$i];
			}
			elsif ($ARGV[$i] eq "-h")
			{
				$i++;
				$host=$ARGV[$i];
			}
		# Ephemeris
			elsif ($ARGV[$i]=~/--ephemeris=*/)
			{
				$ARGV[$i]=~s/--ephemeris=//;
				$ephemeris_path=$ARGV[$i];
			}
			elsif ($ARGV[$i] eq "-e")
			{
				$i++;
				$ephemeris_path=$ARGV[$i];
			}
		# Attitude
			elsif ($ARGV[$i]=~/--attitude=*/)
			{
				$ARGV[$i]=~s/--attitude=//;
				$attitude_path=$ARGV[$i];
			}
			elsif ($ARGV[$i] eq "-a")
			{
				$i++;
				$attitude_path=$ARGV[$i];
			}
		# Initialization
			elsif ($ARGV[$i]=~/--initialize=*/)
			{
				$ARGV[$i]=~s/--initialize=//;
				$initialize_path=$ARGV[$i];
			}
			elsif ($ARGV[$i] eq "-i")
			{
				$i++;
				$initialize_path=$ARGV[$i];
			}
		# Model file
			elsif ($ARGV[$i]=~/--model=*/)
			{
				$ARGV[$i]=~s/--model=//;
				$model_path=$ARGV[$i];
			}
			elsif ($ARGV[$i] eq "-m")
			{
				$i++;
				$model_path=$ARGV[$i];
			}
		# Verbose
			elsif ($ARGV[$i] eq "--verbose")
			{
				$verbose_output=true;
			}
			elsif ($ARGV[$i] eq "-v")
			{
				$verbose_output=true;
			}
	}

# Verify command line arguments
	if ($timestep_skip!=0)
	{
		print "Using a timestep of $timestep_skip seconds.\n\n";
	}
	
	if ($time_limit!=0)
	{
		print "Limiting to first $time_limit seconds in file.\n\n";
	}
	
	print "Using $host as host.\n\n";
	

# Initialize STK connection parameters
	use Stk;
	$port=5001;
		
# Connect to STK and initialize scenario
	print "Establishing connection...  ";
	$connection_established=Connect_to_STK($port,$host,STK);
	if (!$connection_established)
	{
		print "unable to connect to $host:$port.  Aborting\n\n";
		exit;
	}
	else
	{
		print "connected to STK on $host:$port.\n\nSending Connect commands...\n\n";
		
		# Initialize scenario and satellites
			STK_Connect_Command(STK, "Unload / */Satellite/ARES-1", $verbose_output);
			STK_Connect_Command(STK, "New / */Satellite ARES-1", $verbose_output);
			STK_Connect_Command(STK, "Graphics */Satellite/ARES-1 SetColor Green", $verbose_output);
			STK_Connect_Command(STK, "SetUnits / EPSEC ft Deg", $verbose_output);
		
		# Load the model file if specified
			if (!($model_path eq ""))
			{
				if (!(STK_Connect_Command(STK, "VO */Satellite/ARES-1 Model File \"$model_path\"", $verbose_output)))
				{
					print "Error connecting model $model_path to object.\n\n";
				}
				else
				{
					print "Using $model_path as object model.\n\n";
				}
				
				# Attempt to load articulation file
					if ($verbose_output)
					{
						if (!STK_Connect_Command(STK, "VO */Satellite/ARES-1 ReloadArticFile", $verbose_output))
						{
							print "No appropriate model articulation file was found.\n\n";
						}
						else
						{
							print "Reloaded model articulation file.\n\n";
						}
					}
			}
			
		# Initialize the animation
			STK_Connect_Command(STK, "SetAnimation * TimeStep .05", $verbose_output);
			STK_Connect_Command(STK, "Animate * Reset", $verbose_output);
#			STK_Connect_Command(STK, "SetAnimation * StartTime \"21 Sep 2016 18:38:24.9\"", $verbose_output); 
		
		# Enable vectors		
			STK_Connect_Command(STK, "VO */Satellite/ARES-1 SetVectorGeometry Add \"Satellite/ARES-1 Center Point\" TrajectoryType Line ShowCartesian On Color pink", $verbose_output);
			STK_Connect_Command(STK, "VO */Satellite/ARES-1 SetVectorGeometry Modify \"Body Axes\" Show On Color blue", $verbose_output);
			STK_Connect_Command(STK, "VO */Satellite/ARES-1 SetVectorGeometry Modify \"Velocity Vector\" Show On Color red", $verbose_output);
			STK_Connect_Command(STK, "VO */Satellite/ARES-1 SetVectorGeometry Modify \"Velocity Vector\" Show On Color red", $verbose_output);

		# Run the initialization file if specified
			if (!($initialize_path eq ""))
			{
				open(INIT_IN,"< $initialize_path");
				@initialize=<INIT_IN>;
				close(INIT_IN);
				
				print "\nSending custom commands found in $initialize_path...\n";
				foreach $command (@initialize)
				{
					STK_Connect_Command(STK, $command, $verbose_output);		
				}
				print "\n\n";
			}
	}

# Feed input file data to STK
	# Read all data from file
		open(IN,"< $file");
		@data=<IN>;
		close(IN);
	
	# Process each line in file
		$linenum=0;
		foreach $line (@data)
		{
			# Trim whitespace from front and end of line
				$line=~s/\s+//;
				$line=~s/\s+$//;
				
			# If line does contain the 'time' string, extract the key names from the array; this allows the columns to be placed in any order in the output file
				if ($line=~/[tT][iI]/)
				{
					# Split into a set of data by whitespace between column names
						@names=split(/\s+/,$line);
					
					# Make a hash: (row names) => (column #)
						$i=0;
						foreach $name (@names)
						{
							$namesreverse{$name}=$i;
							$i++;
						}
						
					# Determine what can be visualized through STK
						print "\nDetected parameters: ";
						if (!(($namesreverse{"latDeg.C"} eq "") || ($namesreverse{"lonDeg.C"} eq "") || ($namesreverse{"alt_Ft.C"} eq "")))
						{
							print "Maveric-LLA (deg/ft)  ";
							$visualize_maveric_LLA=true;
						}
						if (!(($namesreverse{"betaDeg.C"} eq "") || ($namesreverse{"betaDeg.C"} eq "") || ($namesreverse{"phibkDeg.C"} eq "")))
						{
							print "Maveric-Attitude (deg)  ";
							$visualize_maveric_attitude=true;
						}
						
						if (!(($namesreverse{"ALT"} eq "") || ($namesreverse{"LATD"} eq "") || ($namesreverse{"LOND"} eq "")))
						{
							print "OTIS-LLA (ft/deg)  ";
							$visualize_otis_LLA=true;
						}
						print "\n\n";
				}
			# Otherwise, populate array and send data to STK
				else
				{
					# Split into a set of data by whitespace between columns
						@row=split(/\s+/,$line);
						
					# Extract current number of seconds from string time variable
						@timesplit=split(/e/,$row[$namesreverse{time}]);
						if ($timesplit[1] eq "")
						{
							$currenttime=$timesplit[0];
						}
						else
						{
							$currenttime=$timesplit[0]* (10**($timesplit[1]));
						}
					
					# - If a time limit was specified, time must be less than this limit to continue.
					# - If time between last timestep and current timestep is less than the desired STK timestep, send the data to STK.
					# - Always send the first and last timesteps in file.
						if (($time_limit==0 || $currenttime<=$time_limit) && (($currenttime!=$lasttime && ($currenttime-$lasttime) >= $timestep_skip) || $lasttime eq "" || $linenum==@data-1))
						{
							# Maveric
								# Latitude/Longitude/Altitude
									if ($visualize_maveric_LLA)
									{
										STK_Connect_Command(STK, "SetPosition */Satellite/ARES-1 LLA \"$row[$namesreverse{\"time\"}]\" $row[$namesreverse{\"latDeg.C\"}] $row[$namesreverse{\"lonDeg.C\"}] $row[$namesreverse{\"alt_Ft.C\"}]", $verbose_output);
									}
								# Roll/Pitch/Yaw attitude
									if ($visualize_maveric_attitude)
									{
										STK_Connect_Command(STK, "AddAttitude */Satellite/ARES-1 ECIYPR \"$row[$namesreverse{\"time\"}]\" $row[$namesreverse{\"betaDeg.C\"}] $row[$namesreverse{\"alphaDeg.C\"}] $row[$namesreverse{\"phibkDeg.C\"}]", $verbose_output);
									}
								
							# OTIS
								# Latitude/Longitude/Altitude
									if ($visualize_otis_LLA)
									{
										STK_Connect_Command(STK, "SetPosition */Satellite/ARES-1 LLA \"$row[$namesreverse{\"TIME\"}]\" $row[$namesreverse{\"LATD\"}] $row[$namesreverse{\"LOND\"}] $row[$namesreverse{\"ALT\"}]", $verbose_output);
									}
									
							$lasttime=$currenttime;
						}
				}
				$linenum++;
		}
		
	# Write ephemeris file if specified
		if (!($ephemeris_path eq ""))
		{
			print "\nWriting ephemeris file to $ephemeris_path...\n\n";
			if (!STK_Connect_Command(STK, "ExportDataFile */Satellite/ARES-1 Ephemeris \"$ephemeris_path\" STK J2000 Earth IncludeInterpBound", $verbose_output))
			{
				print "Error writing ephemeris file.  Check to see if the specified directory exists and that a full pathname (directory\file) has been specified.\n\n"
			}
		}
		
	# Write attitude file if specified
		if (!($attitude_path eq ""))
		{
			print "\nWriting attitude file to $attitude_path...\n\n";
			if (!STK_Connect_Command(STK, "ExportDataFile */Satellite/ARES-1 Attitude \"$attitude_path\" Inertial QuatAngVel", $verbose_output))
			{
				print "Error writing attitude file.  Check to see if the specified directory exists and that a full pathname (directory\file) has been specified.\n"
			}
		}
		
		print "\nFinished.\n\n";
